##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import os, zLOG, random, sys
from Globals import package_home, InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from DateTime import DateTime
from copy import deepcopy


factory_type_information = ({'id': 'Encoder Profile',
                            'meta_type': 'Media Manager Encoder Profile',
                            'description': 'Encoder profiles are used to set default values',
                            'product': 'MyMediaManager',
                            'icon': 'encoderprofile_icon.gif',
                            'factory': 'addEncoderProfile',
                            'filter_content_types': 0,
                            'immediate_view': 'encoderprofile_edit_form',
                            'actions':({'name': 'View',
                                        'id': 'view',
                                        'action': 'encoderprofile_view',
                                        'permissions': ('View',),
                                        'category': 'object'},
                                        {'name': 'Edit',
                                        'edit': 'edit',
                                        'action': 'encoderprofile_edit_form',
                                        'permissions': ('Modify portal content',)})
                            },)

class EncoderProfile(PortalContent, DefaultDublinCoreImpl):

    security = ClassSecurityInfo()
    
    meta_type = 'Media Manager Encoder Profile'
    
    def __init__(self, id):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = ''
        self.description = ''
        self.width = 0
        self.height = 0
        self.tool = ''

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, title=None, REQUEST=None):
        """
        edit a profile.
        """
        profile = {}
        if title is not None:
            self.title = title

        if REQUEST is not None:
            profile['properties'] = []
        
            self.encoder = REQUEST.get('encoder','')

            try:
                encodertool = self.restrictedTraverse(self.encoder)
            except:
                raise "Encoder '%s' could not be found" % self.encoder
                
            for property in encodertool.getProfileStructure():
                prop = deepcopy(property)
                p = REQUEST.get(prop['id'], '')
                if prop['type'] == 'int':
                    try:
                        prop['value'] = int(p)
                    except: 
                        raise "The value %s is not an integer" % str(p)
                elif prop['type'] == 'boolean':
                    if str(p).lower() in ('yes','no','true','false'):
                        if p in ('yes','true'):
                            prop['value'] = 1
                        else:
                            prop['value'] = 0
                    try:
                        p = int(p)
                    except: pass
                    if p:
                        prop['value'] = 1
                    else:
                        prop['value'] = 0
                elif prop['type'] in ('string','text','selection'):
                    prop['value'] = p
                elif prop['type'] == 'multiple selection':
                    if type(p) not in (type(()), type([])):
                        p = (p,)
                    prop['value'] = list(p)
                profile['properties'].append(prop)
            
            for prop in profile['properties']:
                setattr(self, prop['id'], prop.get('value',''))

            self.reindexObject()
            msg = "Profile saved"                      
            if REQUEST.has_key('HTTP_REFERER'):
                REQUEST.RESPONSE.redirect("%s/encoderprofile_edit_form?portal_status_message=%s" % (self.absolute_url(), msg.replace(' ', '+')))

    security.declarePublic('getProfile')
    def getProfile(self):
        """
        Returns the profile as a dictionary
        according to <encodertool>.getProfileStructure
        """
        try:
            encodertool = self.restrictedTraverse(self.encoder)
        except:
            raise "Encoder '%s' could not be found" % self.encoder
                
        properties = {}
        for prop in encodertool.getProfileStructure():
            if hasattr(self, prop['id']):
                properties[prop['id']] = getattr(self, prop['id'], '')
            else:
                if prop['type'] in ('selection','multiple selection'):
                    properties[prop['id']] = prop.get('selected', '')
                else:
                    properties[prop['id']] = prop.get('value', '')
        return properties
        
    security.declarePublic('getValue')
    def getValue(self, id):
        """
        Returns the value for id
        """
        try:
            encodertool = self.restrictedTraverse(self.encoder)
        except:
            raise "Encoder '%s' could not be found" % self.encoder
                
        val = None
        for prop in encodertool.getProfileStructure():
            if id == prop['id']:
                if hasattr(self, prop['id']):
                    val = getattr(self, prop['id'], None)
                else:
                    if prop['type'] in ('selection','multiple selection'):
                        expr = prop.get('selected', None)
                        val = encodertool.evaluateExpression(expression=expr, object=self)
                    else:
                        val = prop.get('value', None)
                return val

def addEncoderProfile(self, id, REQUEST=None):
    ob=EncoderProfile(id)
    self._setObject(id, ob)
            
InitializeClass(EncoderProfile)
