# $Id: install.py,v 1.4.6.35 2004/04/03 22:28:46 gitte Exp $
# $Source: /usr/local/cvs_root/MyMediaManager/Extensions/install.py,v $
__version__ = "$Revision: 1.4.6.35 $"[11:-2]

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews, createDirectoryView
from Products.CMFCore.utils import getToolByName, minimalpath
#from Products.CMFBoringProduct import CMFBoringContent, boring_globals
from Products.MyMediaManager import mmm_globals, MediaClip, MasterFile, MediaEvent, Medianet, MediaCinema, MediaSkinExtensions
from Products.ExternalMethod import ExternalMethod
from App.Common import package_home
from cStringIO import StringIO
import string
import zLOG

try:
    from Products.CMFPlone.Portal import PloneSite
    usePlone = 1
except:
    usePlone = 0

try: 
    from Products.CPSDefault.Portal import CPSDefaultSite
    useCPS = 1
except:
    useCPS = 0

out = StringIO()

type_classes = (MediaClip, MasterFile, MediaEvent, MediaCinema, MediaSkinExtensions)
folderish_types = ('Media Event',)
workspace_types = ['Media Event', 'Media Cinema', 'Media Player', 'Media Playlist']

def registerType(self, fti, portal, typestool):
    """
        Register type defined in fti in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if fti['id'] not in typestool.objectIds():
        if useCPS:
            ftiid = fti['id'].replace(' ', '')
            fti['title'] = 'portal_type_%s_title' % ftiid
            fti['cps_is_searchable'] = 1
        cfm = apply(ContentFactoryMetadata, (), fti)
        typestool._setObject(fti['id'], cfm)
        out.write('Registered %s with the types tool\n' % fti['id'])
    else:
        out.write('Object "%s" already existed in the types tool\n' % (fti['id']))
    
    if useCPS:
        out.write('Adding workflow for %s\n' % fti['id'])
        workspace = getattr(portal, 'workspaces', None)
        if workspace:
            wfc = getattr(workspace, '.cps_workflow_configuration', None)
            if wfc:
                chain = 'workspace_content_wf'
                if fti['id'] in folderish_types:
                    chain = 'workspace_folderish_content_wf'
                wfc.manage_addChain(portal_type=fti['id'], chain=chain)
                out.write("Added workspace workflow chain '%s' for %s\n" % (chain, fti['id']))
            else:
                out.write('Could not find workflow configuration for workspaces\n')
        else:
            out.write('Could not find workspaces in CPS site\n')
        if fti['id'] in workspace_types:
            workspacefti = typestool.getTypeInfo('Workspace')
            allowed_types = list(workspacefti.allowed_content_types)
            if fti['id'] not in allowed_types:
                allowed_types.append(fti['id'])
                workspacefti.manage_changeProperties(allowed_content_types=allowed_types)
                out.write('Added %s as allowed type in workspace\n' % fti['id'])
            else:
                out.write('%s already allowed as type in workspace\n' % fti['id'])
            objfti = typestool.getTypeInfo(fti['id'])
            objfti.manage_changeProperties(cps_display_as_document_in_listing=1)
            obj.write('Display %s in document listings\n' % fti['id'])
        else:
            out.write('%s not allowed as type in workspace\n' % fti['id'])
    else:
        out.write('Skipping workflow configuration - not a CPS site\n')

def unregisterType(self, type):
    """
        Un-Register type in the types tool
    """
    typestool = getToolByName(self, 'portal_types')
    if type in typestool.objectIds():
        typestool._delObject(type)
        out.write('Un-Registered %s with the types tool\n' % type)
    else:
        out.write('Object "%s" did not exist in the types tool\n' % type)

def removeSkin(self, skinname):
    """
    Remove a skin from portal_skins
    """
    skinstool = getToolByName(self, 'portal_skins')
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if skinname in path:
            path.remove(skinname)
            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Removed %s from %s skin\n" % (skinname, skin))

    if skinname in skinstool.objectIds():
        skinstool._delObject(skinname)
        out.write('Removed %s from the skinstool\n' % skinname)

def install(self):
    """Register all MMMCore components with the necessary tools"""
    out.write('MyMediaManager "New Generation" Installation\n')
    out.write('===========================\n\n')
    skinstool = getToolByName(self, 'portal_skins')
    typestool = getToolByName(self, 'portal_types')
    actionstool = getToolByName(self, 'portal_actions')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    portalcatalog = getToolByName(self, 'portal_catalog')
    try:
        portal_conf = getToolByName(self, 'portal_controlpanel')
        portal_conf.registerConfiglet('MyMediaManager'
                                    , 'My Media Manager Medianet'
                                    , 'string: ${portal_url}/portal_medianet/medianet_admin_form'
                                    , 'python: "portal_medianet" in portal.objectIds()'
                                    , 'Manage portal'
                                    , 'Products'
                                    , 1
                                    , 'MyMediaManager'
                                    , 'medianet_icon.gif'
                                    , 'This is the medianet where you setup the various media servers'
                                    , None)
        out.write('Added configlet for Medianet\n')
    except:
        out.write('Not a Plone site - no configlet added for Medianet\n')

    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                registerType(self, fti, portalroot, typestool)
        else:
            registerType(self, t, portalroot, typestool)

    tools = {'portal_medianet':'Media Manager Medianet',}

    for tool in tools.keys():
        if not tool in portalroot.objectIds():
            portalroot.manage_addProduct['MyMediaManager'].manage_addTool(tools[tool], None)
            out.write('Created %s\n' % tools[tool])
        else:
            out.write('%s already exists\n' % tools[tool])

    ap = actionstool.listActionProviders()
    for tool in tools.keys():
        if (type(ap) is not type([]) and type(ap) is not type(())) or not tool in ap:
            toolobj = getToolByName(self, tool)
            if hasattr(toolobj, '_actions'):
                try:
                    actionstool.addActionProvider(tool)
                    out.write('Added %s as action provider\n' % tools[tool])
                except:
                    out.write('Failed adding %s as action provider. Please do it manually\n' % tools[tool])
            else:
                out.write('%s provides no actions\n' % tools[tool])
        else:
            out.write('%s already existed as action provider\n' % tools[tool])

    if not 'getMemberPass' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('getMemberPass', 'Returns a members password', 'MyMediaManager.utils', 'getUserPass')
        portalroot._setObject('getMemberPass', i)
        out.write('Created getMemberPass method\n')
    else:
        out.write('getMemberPass method already existed\n')

    if not 'transitions' in portalroot.objectIds():
        i = ExternalMethod.ExternalMethod('transitions', 'Get transitions from portal_workflow', 'MyMediaManager.utils', 'getTransitions')
        portalroot._setObject('transitions', i)
        out.write('Created transitions method\n')
    else:
        out.write('transitions method already existed\n')

    if not 'start' in portalcatalog.Indexes.objectIds():
        portalcatalog.addIndex('start', 'FieldIndex')
        out.write('Added start as FieldIndex\n')
    else:
        out.write('start already exists as FieldIndex\n')

    if not 'end' in portalcatalog.Indexes.objectIds():
        portalcatalog.addIndex('end', 'FieldIndex')
        out.write('Added end as FieldIndex\n')
    else:
        out.write('end already exists as FieldIndex\n')

    if not 'default_cinema' in portalroot.objectIds():
        portalroot.invokeFactory(id='default_cinema', type_name='Media Cinema')
        cinema = getattr(portalroot, 'default_cinema')
        text = open(cinema._default_content_fn).read()
        cinema.edit(title='Default Cinema', text=text)
        out.write('Added default cinema\n')
    else:
        out.write('Cinema already exists\n')

    # Setup the skins
    if 'mmm' not in skinstool.objectIds():
        # We need to add Filesystem Directory Views for any directories
        # in our skins/ directory.  These directories should already be
        # configured.
        fp = minimalpath("%s/zpt_skins" % package_home(mmm_globals))
        createDirectoryView(skinstool, fp, 'mmm')
        #addDirectoryViews(skinstool, 'zpt_skins', fp)
        out.write("Added 'mmm' directory view to portal_skins\n")
    else:
        out.write("'mmm' directory view already exists in portal_skins\n")

    # Now we need to go through the skin configurations and insert
    # 'CMFBoringContent' into the configurations.  Preferably, this
    # should be right before where 'content' is placed.  Otherwise, we
    # append it to the end.
    skins = skinstool.getSkinSelections()
    for skin in skins:
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if 'mmm' not in path:
            try: path.insert(path.index('content'), 'mmm')
            except ValueError:
                path.append('mmm')

            path = string.join(path, ', ')
            # addSkinSelection will replace existing skins as well.
            skinstool.addSkinSelection(skin, path)
            out.write("Added 'mmm' to %s skin\n" % skin)
        else:
            out.write("Skipping %s skin, 'mmm' is already set up\n" % (skin))
    
    if usePlone:
        if 'mmm_plone' not in skinstool.objectIds():
            # We need to add Filesystem Directory Views for any directories
            # in our skins/ directory.  These directories should already be
            # configured.
            fp = minimalpath("%s/plone_skins" % package_home(mmm_globals))
            createDirectoryView(skinstool, fp, 'mmm_plone')
            #addDirectoryViews(skinstool, 'zpt_skins', fp)
            out.write("Added 'mmm_plone' directory view to portal_skins\n")
        else:
            out.write("'mmm_plone' directory view already exists in portal_skins\n")

        # Now we need to go through the skin configurations and insert
        # 'CMFBoringContent' into the configurations.  Preferably, this
        # should be right before where 'content' is placed.  Otherwise, we
        # append it to the end.
        skins = skinstool.getSkinSelections()
        for skin in skins:
            path = skinstool.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))
            if 'mmm_plone' not in path:
                try: path.insert(path.index('mmm'), 'mmm_plone')
                except ValueError:
                    path.append('mmm_plone')

                path = string.join(path, ', ')
                # addSkinSelection will replace existing skins as well.
                skinstool.addSkinSelection(skin, path)
                out.write("Added 'mmm_plone' to %s skin\n" % skin)
            else:
                out.write("Skipping %s skin, 'mmm_plone' is already set up\n" % (skin))
    
    if useCPS:
        if 'mmm_cps' not in skinstool.objectIds():
            # We need to add Filesystem Directory Views for any directories
            # in our skins/ directory.  These directories should already be
            # configured.
            fp = minimalpath("%s/cps_skins" % package_home(mmm_globals))
            createDirectoryView(skinstool, fp, 'mmm_cps')
            #addDirectoryViews(skinstool, 'zpt_skins', fp)
            out.write("Added 'mmm_cps' directory view to portal_skins\n")
        else:
            out.write("'mmm_cps' directory view already exists in portal_skins\n")

        # Now we need to go through the skin configurations and insert
        # 'CMFBoringContent' into the configurations.  Preferably, this
        # should be right before where 'content' is placed.  Otherwise, we
        # append it to the end.
        skins = skinstool.getSkinSelections()
        for skin in skins:
            path = skinstool.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))
            if 'mmm_cps' not in path:
                try: path.insert(path.index('mmm'), 'mmm_cps')
                except ValueError:
                    path.append('mmm_cps')

                path = string.join(path, ', ')
                # addSkinSelection will replace existing skins as well.
                skinstool.addSkinSelection(skin, path)
                out.write("Added 'mmm_cps' to %s skin\n" % skin)
            else:
                out.write("Skipping %s skin, 'mmm_cps' is already set up\n" % (skin))
            
    return out.getvalue()

def uninstall(self):
    """
    Remove skins, types and special objects
    """
    out.write('MyMediaManager Un-Installation\n')
    out.write('===========================\n\n')
    portalroot = getToolByName(self, 'portal_url').getPortalObject()
    actionstool = getToolByName(self, 'portal_actions')
    for c in type_classes:
        t = c.factory_type_information
        if type(t) == type(()):
            for fti in t:
                unregisterType(self, fti['id'])
        else:
            unregisterType(self, t['id'])

    removeSkin(self, 'mmm')
    
    try:
        portal_conf = getToolByName(self, 'portal_controlpanel')
        portal_conf.unregisterConfiglet('MyMediaManager')
        portal_conf.unregisterConfiglet('My Media Manager')
        out.write('Removed configlet for Medianet\n')
    except:
        out.write('Not a Plone site - no configlet removed for Medianet\n')

    tools = {'portal_medianet':'Media Manager Medianet',}

    for tool in tools.keys():
        if tool in portalroot.objectIds():
            portalroot._delObject(tool)
            out.write('Removed %s\n' % tools[tool])
        else:
            out.write("%s didn't exists\n" % tools[tool])

    ap = actionstool.listActionProviders()
    for tool in tools.keys():
        if (type(ap) is not type([]) and type(ap) is not type(())) or tool in ap:
            try:
                    actionstool.deleteActionProvider(tool)
                    out.write('Remoced %s as action provider\n' % tools[tool])
            except:
                    out.write('Failed removing %s as action provider. Please do it manually\n' % tools[tool])
        else:
            out.write("%s didn't existed as action provider\n" % tools[tool])

    return out.getvalue()
