from types import *
from math import *
from Products.CMFCore.utils import getToolByName
from DateTime import DateTime
from Globals import DTMLFile, package_home
from AccessControl import ClassSecurityInfo

import Globals, os
import __builtin__
import zLOG

_dtmldir = os.path.join( package_home( globals() ), 'dtml' )

webcolors = ('aqua','black','blue','fuchsia','gray','green','lime','maroon','navy','olive','purple','red','silver','teal','white','yellow')
audioformats = {'voice': 'Voice only',
	 	'background': 'Voice with background music',
		'music': 'Music',
		'stereo': 'Stereo music'}
videoqualities = {'normal': 'Normal motion',
		  'smooth': 'Smoothest motion',
		  'sharp': 'Sharpest image',
		  'slides': 'Slide show'}
audienceratings = {'general': 'General, all ages',
		   'parent': 'Parental guidance',
		   'supervision': 'Adult supervision required',
		   'adults': 'Adults only'}

bandwidths = [{'28kmodem': '28.8kbs modem'}
            , {'56kmodem': '56.6kbs modem'}
            , {'singleisdn': 'SingleISDN'}
            , {'dualisdn': 'DualISDN'}
            , {'dsl': '150LAN'}
            , {'256k': '256ADSL'}
            , {'384k': '384ADSL'}
            , {'512k': '512ADSL'}
            ]
           
videoinputs = {'y/c': 'Y/C',
	       'sdi': 'SDI',
	       'composite': 'Composite',
	       'component': 'Component'}
videostandards = {'ntsc': 'NTSC',
		  'pal': 'PAL'}
audioinputs = {'aesebu': 'AESEBU',
	       'balanced': 'Balanced',
	       'unbalanced': 'Unbalanced',
	       'sdi': 'SDI'}

class Playable:
   """
     All classes that whant to be playable by either WM or Real
     should implement this
   """

   #embedPlayer = DTMLFile('dtml/embedPlayer', globals())

   def lookupPlayerSource(self):
     return "someRam.rpm"


class TimeStamp:
    """
    Class for simpel timestamps with milliseconds
    """
    security = ClassSecurityInfo()

    _millitime = 0L

    def __init__(self, *args):
        """Return a new timestamp"""

        ac=len(args)
        day,hr,min,sec,milli=0L,0L,0L,0L,0L

        #zLOG.LOG('TimeStamp', zLOG.INFO, '__init__: args: %d %s' % (ac, str(args)))

        if ac and args[0]==None: return
        elif not args:
            milli = 0
        elif ac==1:
            arg=args[0]

            if arg=='':
                raise "Invalid timestamp string", arg

            if type(arg)==StringType:
                day,hr,min,sec,milli = self._parse(arg)
            else:
                milli = arg
        elif ac==2:
            sec,milli=args

        elif ac==3:
            min,sec,milli=args

        elif ac==4:
            hr,min,sec,milli=args

        elif ac==5:
            day,hr,min,sec,milli=args

        else:
            raise "TimestampError","Too many arguments"

        #zLOG.LOG("TimeStamp", 0, "%.2d:%2.2d:%2.2d:%2.2d.%3.3d" % (day,hr,min,sec,milli))
        self._millitime = day*86400000L
        self._millitime = self._millitime + hr*3600000L
        self._millitime = self._millitime + min*60000L
        self._millitime = self._millitime + sec*1000L
        self._millitime = self._millitime + milli

    def _parse(self, string):
        i,l = 0,len(string)-1
        part = ''
        parts = [0L, 0L, 0L, 0L, 0L]

        while l >= 0 and i<5:
            if string[l] in ':., ':
                parts[i] = int(part)
                i = i+1
                part = ''
            else:
                part = string[l] + part

            l = l -1

        if i<5:
            parts[i] = int(part)

        return parts[4],parts[3],parts[2],parts[1],parts[0]

    def __str__(self):
        """Convert a Timestamp to a string"""
        try:
            milli = fmod(self._millitime, 1000)
            sec = fmod(self._millitime, 60000) / 1000
            min = floor (fmod(self._millitime, 3600000) / 60000)
            hr = 0L + self._millitime / 3600000
            return "%.2d:%2.2d:%2.2d.%3.3d" % (hr, min, sec, milli)
        except:
            return ''

    def __add__(self, other):
        if hasattr(other, '_millitime'):
            return self.__class__(self._millitime+other._millitime)
        o = long(other)
        return self.__class__(self._millitime+o)

    __radd__=__add__

    def __sub__(self, other):
        o = long(other)
        return self.__class__(self._millitime-o)

    def sub(self, other):
        o = long(other)
        return self.__class__(self._millitime-o)

    def __long__(self):
        return self._millitime

    def __float__(self):
        return self._millitime
    
    security.declarePublic('millis')
    def millis(self):
        return self._millitime
     
    def greaterThan(self,t):
        try:    return (self.millis() > t.millis())
        except: return (self.millis() > t)

    def greaterThanEqualTo(self, t):
        try:    return (self.millis() >= t.millis())
        except: return (self.millis() >= t)

    def equalTo(self, t):
        try:    return (self.millis() == t.millis())
        except: return (self.millis() == t)

    def notEqualTo(self, t):
        try:    return (self.millis() != t.millis())
        except: return (self.millis() != t)

    def lessThan(self,t):
        try:    return (self.millis() < t.millis())
        except: return (self.millis() < t)

    def lessThanEqualTo(self, t):
        try:    return (self.millis() <= t.millis())
        except: return (self.millis() <= t)


def issubclass ( class1, class2):
   if type (class1) is ClassType and type(class2) is ClassType:
     return __builtin__.issubclass(class1, class2)

   elif hasattr(class1, '__bases__') and hasattr(class2, '__bases__'):
     stack = [class1]
     while stack:
       if stack[0] is class2:
         return 1
       stack.extend(list(stack[0].__bases__))
       del stack[0]
     else:
       return 0

   else:
     raise TypeError, "arguments must be class or ExtensionClass objects"

def isinstance (object, klass):
   if type(klass) is TypeType:
     return __builtin__.isinstance(object, klass)
   
   elif hasattr(object, '__class__'):
#     zLOG.LOG("General.isinstance", 0, "Class: %s" % (object.__class__))
     return issubclass(object.__class__, klass)
   else:
     return 0

def getUniqueID( prefix = None ):
    if not prefix:
        prefix = getToolByName(self, 'portal_membership').getAuthenticatedMember().getUserName()
    time=str(DateTime().strftime('%m%d%y%H%M%S'))
    fn="%s_%s"%(prefix,time)
    return fn

#t = TimeStamp()
#if (isinstance(t, TimeStamp)):
#     print "Is TimeStamp\n"
#
#t = "Test"
#if (isinstance(t, TimeStamp)):
#     print "Is still TimeStamp\n"
